{********************************************************************}
{                                                                    }
{       SAP Data Viewer                                              }
{                                                                    }
{       Copyright (c) 2004-2007 Software Union G.m.b.H.              }
{       ALL RIGHTS RESERVED                                          }
{                                                                    }
{   The entire contents of this file is protected by                 }
{   International Copyright Laws. Unauthorized reproduction,         }
{   reverse-engineering, and distribution of all or any portion of   }
{   the code contained in this file is strictly prohibited and may   }
{   result in severe civil and criminal penalties and will be        }
{   prosecuted to the maximum extent possible under the law.         }
{                                                                    }
{   RESTRICTIONS                                                     }
{                                                                    }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES            }
{   (DCU, OBJ, DLL, ETC.) ARE CONFIDENTIAL AND PROPRIETARY TRADE     }
{   SECRETS OF SOFTWARE UNION G.M.B.H. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE SAP DATA VIEWER AND ALL               }
{   ACCOMPANYING SOFTWARE LIBRARIES AS PART OF AN EXECUTABLE         }
{   PROGRAM ONLY.                                                    }
{                                                                    }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED       }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE         }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE        }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT   }
{   AND PERMISSION FROM SOFTWARE UNION G.M.B.H.                      }
{                                                                    }
{********************************************************************}
//PROFILE-NO
// Note:
// =====
// This application makes extensive use of the DevExpress components,
// available from www.devexpress.com. However, since they are only used
// for UI enhancement, it is no problem to replace them with their
// normal VCL counterparts (and some hundreds of hours work).

unit sapdata;

interface

uses
  Messages, Classes, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, XPMan, Graphics,
  cxGraphics, cxStyles, cxControls, dxStatusBar, cxProgressBar, cxTextEdit, cxTL,
  cxContainer, cxEdit, cxCustomData,  cxMaskEdit, cxCalendar, cxCurrencyEdit,
  cxInplaceContainer, sapdef, cxLabel, cxSpinEdit, cxTimeEdit;

type
  TfrmMain = class(TForm)
    FileOpenDialog: TOpenDialog;
    EditFileName: TEdit;
    btnFileBrowse: TButton;
    btnRead: TButton;
    pnlTree: TPanel;
    XPManifest: TXPManifest;
    TreeList: TcxTreeList;
    ColumnNames: TcxTreeListColumn;
    ColumnData: TcxTreeListColumn;
    lblInfo: TLabel;
    StatusBar: TdxStatusBar;
    ProgressFileRead: TcxProgressBar;
    chkProcessEmpty: TCheckBox;
    ColumnInfo: TcxTreeListColumn;
    StatusBarParFilesInfoContainer: TdxStatusBarContainerControl;
    lblParFilesInfo: TcxLabel;
    procedure btnFileBrowseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure IDocFileProgress(iStepBy, iMax : integer);
    // For File Drag&Drop
    procedure WMDROPFILES(var Msg: TMessage);
    procedure LBWindowProc(var Message: TMessage);
    procedure EMailDblClick(Sender: TObject);
    procedure btnReadClick(Sender: TObject);
    procedure TreeListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TreeListResize(Sender: TObject);
    procedure ColumnDataGetEditProperties(
      Sender: TcxTreeListColumn; ANode: TcxTreeListNode;
      var EditProperties: TcxCustomEditProperties);
    procedure EditFileNameKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure TreeListCustomDrawCell(Sender: TObject; ACanvas: TcxCanvas;
      AViewInfo: TcxTreeListEditCellViewInfo; var ADone: Boolean);
  private
    { Private declarations }
    DateEditProperties : TcxDateEditProperties;
    NumberEditProperties : TcxCurrencyEditProperties;
    StringEditProperties : TcxTextEditProperties;
    TimeEditProperties : TcxTimeEditProperties;
    FilterValue : string;
    procedure InitSapDefs;
    function InitSapDefsFromFile : boolean;
    procedure CheckFileParams(const sWildCard : string);
    procedure PutIDocInTreeView(const IDocInput : TIDoc; const BaseName : string = 'IDOC');
    procedure SaveToFile(Sender: TcxTreeList);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses dateutils, Windows, SysUtils, shellapi, IniFiles, Variants, SapParse;

var OldLBWindowProc : TWndMethod;
   IDoc : TIDoc;      // Template IDoc containing all Segment definitions

//------------------------------------------------------------------------------
procedure TfrmMain.btnFileBrowseClick(Sender: TObject);
//------------------------------------------------------------------------------
// Execute the file selection dialog
//------------------------------------------------------------------------------
begin
   if FileOpenDialog.Execute then
      EditFileName.Text := FileOpenDialog.FileName;

end;

//------------------------------------------------------------------------------
procedure TfrmMain.FormCreate(Sender: TObject);
//------------------------------------------------------------------------------
// Form Creation, processing of Parameters
//------------------------------------------------------------------------------
begin
   InitSapDefs;

   DateEditProperties   := TcxDateEditProperties.Create(self);
   NumberEditProperties := TcxCurrencyEditProperties.Create(self);
   StringEditProperties := TcxTextEditProperties.Create(self);
   TimeEditProperties   := TcxTimeEditProperties.Create(self);

   OldLBWindowProc := TreeList.WindowProc; // store default WindowProc
   TreeList.WindowProc := LBWindowProc;    // replace default WindowProc
   DragAcceptFiles(TreeList.Handle, True); // now Treelist accepts dropped files

   if ParamStr(1) <> '' then
   begin
      CheckFileParams(ParamStr(1));
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.btnReadClick(Sender: TObject);
//------------------------------------------------------------------------------
// Read the data and place the into the treeview
//------------------------------------------------------------------------------
var
    IDocInput : TIDoc;
    starttime : TDateTime;
begin
   if not FileExists(EditFileName.Text) then
      MessageDlg('File not found', mtError, [mbOk], 0)
   else
   begin
      try
         btnRead.Enabled := false;
         ProgressFileRead.Position := 0;
         ProgressFileRead.Properties.EndColor := clNavy;
         ProgressFileRead.Show;
         Application.ProcessMessages;
         starttime := Now;
         // Handler which is called while processing the input file
         IDoc.OnReadFromFile := IDocFileProgress;
         // Load the file into the IDoc Object
         IDocInput := IDoc.LoadFromFile(EditFileName.Text);
         lblInfo.Caption := 'IDoc read '+FormatFloat('0.00000',SecondSpan(Now,starttime))+'s';
         // Write IDoc into Treeview
         PutIDocInTreeView(IDocInput, EditFileName.Text);
         TreeList.SetFocus;
         IDocInput.Free;
      finally
         // Reset Progressbar and button
         ProgressFileRead.Position := ProgressFileRead.Properties.Min;
         ProgressFileRead.Hide;
         btnRead.Enabled := true;
      end;
   end;
end;

var
  ProgressModulo : integer;
  ProgressNewPosition : extended;
  ProgressOldPosition : extended;
//------------------------------------------------------------------------------
procedure TfrmMain.IDocFileProgress(iStepBy, iMax: integer);
//------------------------------------------------------------------------------
// Notification which is called while processing the IDOC
// iStepBy : < 0 = Start
//           > 0 = Number of lines processed in this step
//------------------------------------------------------------------------------
begin
   if iStepBy < 0 then begin
      ProgressFileRead.Properties.Max := iMax;
      ProgressFileRead.Properties.Min := 0;
      ProgressFileRead.Position := 0;
      ProgressOldPosition := 0;
      ProgressNewPosition := 0;
      ProgressModulo := trunc(iMax/200);
   end
   else
   begin
      ProgressNewPosition := ProgressNewPosition+iStepBy;
      if ProgressNewPosition-ProgressOldPosition >= ProgressModulo then
      begin
         ProgressFileRead.Position := ProgressNewPosition;
         ProgressOldPosition := ProgressNewPosition;
         Application.ProcessMessages;
      end;
   end;

end;

//------------------------------------------------------------------------------
procedure TfrmMain.PutIDocInTreeView(const IDocInput: TIDoc; const BaseName : string);
//------------------------------------------------------------------------------
// Transfer all data from the IDoc Object into the Treeview
//------------------------------------------------------------------------------
var
   iRecord, iField : integer;
   FileNode, SegmentNode, FieldNode : TcxTreeListNode;
begin
   if TreeList.ColumnCount = 0 then
   begin
      with TreeList.CreateColumn do
      begin
         Position.BandIndex := 0;
         Width := TreeList.ClientWidth;
      end;

      with TreeList.CreateColumn do
      begin
         Position.BandIndex := 1;
         Width := TreeList.ClientWidth;
      end;
   end;

   ProgressFileRead.Properties.EndColor := clYellow;
   IDocFileProgress(-1, IDocInput.RecordCount);

   TreeList.BeginUpdate;
   try
      FileNode := TreeList.AddChildFirst(TreeList.Nodes.Root);
      FileNode.AssignValues([BaseName, IDocInput.Delimiter]);
      for iRecord := 0 to IDocInput.RecordCount -1 do
      begin
         IDocFileProgress(1, 0);
         with IDocInput.Segments[iRecord] do
         begin

            SegmentNode := FileNode.AddChild;

            if Name = 'EDI_DC40' then
               // 1. Record should be EDI_DC40
               if trim(FieldByName('CIMTYP').AsString) = '' then
                  SegmentNode.AssignValues([Format('%s [%s.%s]', [Name, trim(FieldByName('IDOCTYP').AsString), trim(FieldByName('MESTYP').AsString)]), FieldByName('DocNum').AsInteger])
               else
                  SegmentNode.AssignValues([Format('%s [%s.%s.%s]', [Name, trim(FieldByName('IDOCTYP').AsString), trim(FieldByName('MESTYP').AsString), trim(FieldByName('CIMTYP').AsString)]), FieldByName('DocNum').AsInteger])
            else
            begin
               // All others have segment numbers
               SegmentNode.AssignValues([Name, FieldByName('SegNum').AsInteger]);
            end;

            for iField := 0 to FieldCount-1 do
            begin
               with Fields[iField] do
               begin
                  // Decide how to handle empty fields
                  if (Trim(Data) > '') or chkProcessEmpty.Checked then
                  begin
                     FieldNode := SegmentNode.AddChild;
                     case FieldType of
                        ftDate:
                           begin
                              if AsDateTime = MinDateTime then
                                 FieldNode.AssignValues([Name, Null, IDocInput.Segments[iRecord].Name])
                              else
                                 FieldNode.AssignValues([Name, AsDateTime, IDocInput.Segments[iRecord].Name]);
                           end;
                        ftTime :
                           begin
                              if AsTime = MinDateTime then
                                 FieldNode.AssignValues([Name, Null, IDocInput.Segments[iRecord].Name])
                              else
                                 FieldNode.AssignValues([Name, AsTime, IDocInput.Segments[iRecord].Name]);
                           end;
                        ftInteger : FieldNode.AssignValues([Name, AsInteger, IDocInput.Segments[iRecord].Name]);
                        ftFloat : FieldNode.AssignValues([Name, AsFloat, IDocInput.Segments[iRecord].Name]);
                     else
                        FieldNode.AssignValues([Name, TrimRight(Data), IDocInput.Segments[iRecord].Name]);
                     end;
                  end;
               end;
            end;
         end;
      end;
   finally
      TreeList.EndUpdate;
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.InitSapDefs;
//------------------------------------------------------------------------------
// Generate an IDoc structure containing all needed segment types
// Of course, not all possible segments and fields are contained,
// but can easyliy defined and / or enhanced.
//
// There is also the possibility to read the definitions from a
// WE60 .par file using a simple parser.
//------------------------------------------------------------------------------
var
   CreateHeader : boolean;
   CreateDefaults : boolean;
begin
   // Create the Template IDOC
   // IDoc := TIDoc.Create;
   IDoc := TIDoc.CreateIndexed;

   // This tells the IDOC-Classes not to crash if an undefined
   // Segment is read. The Segment will then be created
   // automatically from whatever is found in it's name field
   // All Data go into a full-length SDATA field.
   IDoc.AutoCreateSegments := true;

   // Don't use hard coded standard definitions if configuration
   // can be read from files
   CreateDefaults := true;
   CreateHeader := true;

   if InitSapDefsFromFile then
   begin
      CreateDefaults := false;
      // If there exists already a control record, it should not be created
      if IDoc.SegmentIndex('EDI_DC40') > -1 then
         CreateHeader := false;
   end;

   if CreateHeader then
   begin
      // Controlrecord, create WITHOUT header |
      //                                      v
      with IDoc.Add(TIDocRecord.Create('EDI_DC40', false)) do begin
         Add(TIDocField.create('TABNAM',  10));
         Add(TIDocField.create('MANDT' ,   3));
         Add(TIDocField.create('DOCNUM',  16));
         Add(TIDocField.create('DOCREL',   4));
         Add(TIDocField.create('STATUS',   2));
         Add(TIDocField.create('DIRECT',   1));
         Add(TIDocField.create('OUTMOD',   1));
         Add(TIDocField.create('EXPRSS',   1));
         Add(TIDocField.create('TEST'  ,   1));
         Add(TIDocField.create('IDOCTY',  30));
         Add(TIDocField.create('CIMTYP',  30));
         Add(TIDocField.create('MESTYP',  30));
         Add(TIDocField.create('MESCOD',   3));
         Add(TIDocField.create('MESFCT',   3));
         Add(TIDocField.create('STD'   ,   1));
         Add(TIDocField.create('STDVRS',   6));
         Add(TIDocField.create('STDMES',   6));
         Add(TIDocField.create('SNDPOR',  10));
         Add(TIDocField.create('SNDPRT',   2));
         Add(TIDocField.create('SNDPFC',   2));
         Add(TIDocField.create('SNDPRN',  10));
         Add(TIDocField.create('SNDSAD',  21));
         Add(TIDocField.create('SNDLAD',  70));
         Add(TIDocField.create('RCVPOR',  10));
         Add(TIDocField.create('RCVPRT',   2));
         Add(TIDocField.create('RCVPFC',   2));
         Add(TIDocField.create('RCVPRN',  10));
         Add(TIDocField.create('RCVSAD',  21));
         Add(TIDocField.create('RCVLAD',  70));
         Add(TIDocField.create('CREDAT',   8, ftDate));
         Add(TIDocField.create('CRETIM',   6, ftTime));
         Add(TIDocField.create('REFINT',  14));
         Add(TIDocField.create('REFGRP',  14));
         Add(TIDocField.create('REFMES',  14));
         Add(TIDocField.create('ARCKEY',  70));
         Add(TIDocField.create('SERIAL',  20));
      end;
   end;



   // If we have read something from par files
   if not CreateDefaults then
      exit;
   StatusBar.Panels[1].Visible := true;
   lblParFilesInfo.Caption := 'Auto';
   lblParFilesInfo.Hint := 'Using internal structure definitions';

   with IDoc.Add(TIDocRecord.Create('E2EDL20002')) do begin
      Add(TIDocField.Create('VBELN'          ,  10));
      Add(TIDocField.Create('VSTEL'          ,   4));
      Add(TIDocField.Create('VKORG'          ,   4));
      Add(TIDocField.Create('LSTEL'          ,   2));
      Add(TIDocField.Create('VKBUR'          ,   4));
      Add(TIDocField.Create('LGNUM'          ,   3));
      Add(TIDocField.Create('ABLAD'          ,  25));
      Add(TIDocField.Create('INCO1'          ,   3));
      Add(TIDocField.Create('INCO2'          ,  28));
      Add(TIDocField.Create('ROUTE'          ,   6));
      Add(TIDocField.Create('VSBED'          ,   2));
      Add(TIDocField.Create('BTGEW'          ,  17, ftFloat, 3));
      Add(TIDocField.Create('NTGEW'          ,  15, ftFloat, 3));
      Add(TIDocField.Create('GEWEI'          ,   3));
      Add(TIDocField.Create('VOLUM'          ,  15, ftFloat, 3));
      Add(TIDocField.Create('VOLEH'          ,   3));
      Add(TIDocField.Create('ANZPK'          ,   5));
      Add(TIDocField.Create('BOLNR'          ,  35));
      Add(TIDocField.Create('TRATY'          ,   4));
      Add(TIDocField.Create('TRAID'          ,  20));
      Add(TIDocField.Create('XABLN'          ,  10));
      Add(TIDocField.Create('LIFEX'          ,  35));
      Add(TIDocField.Create('PARID'          ,  35));
      Add(TIDocField.Create('PODAT'          ,   8, ftDate));
      Add(TIDocField.Create('POTIM'          ,   6));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL18')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('PARAM'          ,  20));
   end;

   With IDoc.Add(TIDocRecord.Create('E2ADRM1001')) do begin
      Add(TIDocField.Create('PARTNER_Q'      ,   3));
      Add(TIDocField.Create('ADDRESS_T'      ,   1));
      Add(TIDocField.Create('PARTNER_ID'     ,  17));
      Add(TIDocField.Create('JURISDIC'       ,  17));
      Add(TIDocField.Create('LANGUAGE'       ,   2));
      Add(TIDocField.Create('FORMOFADDR'     ,  15));
      Add(TIDocField.Create('NAME1'          ,  40));
      Add(TIDocField.Create('NAME2'          ,  40));
      Add(TIDocField.Create('NAME3'          ,  40));
      Add(TIDocField.Create('NAME4'          ,  40));
      Add(TIDocField.Create('NAME_TEXT'      ,  50));
      Add(TIDocField.Create('NAME_CO'        ,  40));
      Add(TIDocField.Create('LOCATION'       ,  40));
      Add(TIDocField.Create('BUILDING'       ,  10));
      Add(TIDocField.Create('FLOOR'          ,  10));
      Add(TIDocField.Create('ROOM'           ,  10));
      Add(TIDocField.Create('STREET1'        ,  40));
      Add(TIDocField.Create('STREET2'        ,  40));
      Add(TIDocField.Create('STREET3'        ,  40));
      Add(TIDocField.Create('HOUSE_SUPL'     ,   4));
      Add(TIDocField.Create('HOUSE_RANG'     ,  10));
      Add(TIDocField.Create('POSTL_COD1'     ,  10));
      Add(TIDocField.Create('POSTL_COD3'     ,  10));
      Add(TIDocField.Create('POSTL_AREA'     ,  15));
      Add(TIDocField.Create('CITY1'          ,  40));
      Add(TIDocField.Create('CITY2'          ,  40));
      Add(TIDocField.Create('POSTL_PBOX'     ,  10));
      Add(TIDocField.Create('POSTL_COD2'     ,  10));
      Add(TIDocField.Create('POSTL_CITY'     ,  40));
      Add(TIDocField.Create('TELEPHONE1'     ,  30));
      Add(TIDocField.Create('TELEPHONE2'     ,  30));
      Add(TIDocField.Create('TELEFAX'        ,  30));
      Add(TIDocField.Create('TELEX'          ,  30));
      Add(TIDocField.Create('E_MAIL'         ,  70));
      Add(TIDocField.Create('COUNTRY1'       ,   3));
      Add(TIDocField.Create('COUNTRY2'       ,   3));
      Add(TIDocField.Create('REGION'         ,   3));
      Add(TIDocField.Create('COUNTY_COD'     ,   3));
      Add(TIDocField.Create('COUNTY_TXT'     ,  25));
      Add(TIDocField.Create('TZCODE'         ,   6));
      Add(TIDocField.Create('TZDESC'         ,  35));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDT13001')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('VSTZW'          ,   4));
      Add(TIDocField.Create('VSTZW_BEZ'      ,  20));
      Add(TIDocField.Create('NTANF'          ,   8));
      Add(TIDocField.Create('NTANZ'          ,   6));
      Add(TIDocField.Create('NTEND'          ,   8));
      Add(TIDocField.Create('NTENZ'          ,   6));
      Add(TIDocField.Create('TZONE_BEG'      ,   6));
      Add(TIDocField.Create('ISDD'           ,   8));
      Add(TIDocField.Create('ISDZ'           ,   6));
      Add(TIDocField.Create('IEDD'           ,   8));
      Add(TIDocField.Create('IEDZ'           ,   6));
      Add(TIDocField.Create('TZONE_END'      ,   6));
      Add(TIDocField.Create('VORNR'          ,   4));
      Add(TIDocField.Create('VSTGA'          ,   4));
      Add(TIDocField.Create('VSTGA_BEZ'      ,  20));
      Add(TIDocField.Create('EVENT'          ,  10));
      Add(TIDocField.Create('EVENT_ALI'      ,  20));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL24004')) do begin
      Add(TIDocField.Create('POSNR'          ,   6));
      Add(TIDocField.Create('MATNR'          ,  18));
      Add(TIDocField.Create('MATWA'          ,  18));
      Add(TIDocField.Create('ARKTX'          ,  40));
      Add(TIDocField.Create('ORKTX'          ,  40));
      Add(TIDocField.Create('SUGRD'          ,   4));
      Add(TIDocField.Create('SUDRU'          ,   1));
      Add(TIDocField.Create('MATKL'          ,   9));
      Add(TIDocField.Create('WERKS'          ,   4));
      Add(TIDocField.Create('LGORT'          ,   4));
      Add(TIDocField.Create('CHARG'          ,  10));
      Add(TIDocField.Create('KDMAT'          ,  22));
      Add(TIDocField.Create('LFIMG'          ,  15));
      Add(TIDocField.Create('VRKME'          ,   3));
      Add(TIDocField.Create('LGMNG'          ,  15));
      Add(TIDocField.Create('MEINS'          ,   3));
      Add(TIDocField.Create('NTGEW'          ,  15));
      Add(TIDocField.Create('BRGEW'          ,  15));
      Add(TIDocField.Create('GEWEI'          ,   3));
      Add(TIDocField.Create('VOLUM'          ,  15));
      Add(TIDocField.Create('VOLEH'          ,   3));
      Add(TIDocField.Create('LGPBE'          ,  10));
      Add(TIDocField.Create('HIPOS'          ,   6));
      Add(TIDocField.Create('HIEVW'          ,   1));
      Add(TIDocField.Create('LADGR'          ,   4));
      Add(TIDocField.Create('TRAGR'          ,   4));
      Add(TIDocField.Create('VKBUR'          ,   4));
      Add(TIDocField.Create('VKGRP'          ,   3));
      Add(TIDocField.Create('VTWEG'          ,   2));
      Add(TIDocField.Create('SPART'          ,   2));
      Add(TIDocField.Create('GRKOR'          ,   3));
      Add(TIDocField.Create('EAN11'          ,  18));
      Add(TIDocField.Create('SERNR'          ,   8));
      Add(TIDocField.Create('AESKD'          ,  17));
      Add(TIDocField.Create('EMPST'          ,  25));
      Add(TIDocField.Create('MFRGR'          ,   8));
      Add(TIDocField.Create('VBRST'          ,  14));
      Add(TIDocField.Create('LABNK'          ,  17));
      Add(TIDocField.Create('ABRDT'          ,   8));
      Add(TIDocField.Create('MFRPN'          ,  40));
      Add(TIDocField.Create('MFRNR'          ,  10));
      Add(TIDocField.Create('ABRVW'          ,   3));
      Add(TIDocField.Create('KDMAT35'        ,  35));
      Add(TIDocField.Create('KANNR'          ,  35));
      Add(TIDocField.Create('POSEX'          ,   6));
      Add(TIDocField.Create('LIEFFZ'         ,  17));
      Add(TIDocField.Create('USR01'          ,  35));
      Add(TIDocField.Create('USR02'          ,  35));
      Add(TIDocField.Create('USR03'          ,  35));
      Add(TIDocField.Create('USR04'          ,  35));
      Add(TIDocField.Create('USR05'          ,  35));
   end;

   // Segment: E2EDL35
   with IDoc.Add(TIDocRecord.Create('E2EDL35')) do
   begin
      Add(TIDocField.Create('STAWN'           ,   17, ftString));  // Commodity code / Import code number for foreign trade
      Add(TIDocField.Create('EXPRF'           ,    5, ftString));  // Export/Import procedure for foreign trade (5 digits)
      Add(TIDocField.Create('EXART'           ,    2, ftString));  // Business transaction type for foreign trade
      Add(TIDocField.Create('HERKL'           ,    3, ftString));  // Country of origin of the material
      Add(TIDocField.Create('HERKR'           ,    3, ftString));  // Region of origin of material (non-preferential origin)
      Add(TIDocField.Create('GRWRT'           ,   15, ftString));  // Statistical value
      Add(TIDocField.Create('PREFE'           ,    1, ftString));  // Preference indicator in export/import
   end;


   with IDoc.Add(TIDocRecord.Create('Z2Y11M_DELVRY03_ITEM_MVT000')) do begin
      Add(TIDocField.Create('LIWERK'         ,   3));
      Add(TIDocField.Create('ABLORT'         ,   2));
      Add(TIDocField.Create('INSMK01'        ,   1));
      Add(TIDocField.Create('BWART01'        ,   3));
      Add(TIDocField.Create('GRUND01'        ,   4));
      Add(TIDocField.Create('INSMK02'        ,   1));
      Add(TIDocField.Create('BWART02'        ,   3));
      Add(TIDocField.Create('GRUND02'        ,   4));
      Add(TIDocField.Create('INSMK03'        ,   1));
      Add(TIDocField.Create('BWART03'        ,   3));
      Add(TIDocField.Create('GRUND03'        ,   4));
      Add(TIDocField.Create('INSMK04'        ,   1));
      Add(TIDocField.Create('BWART04'        ,   3));
      Add(TIDocField.Create('GRUND04'        ,   4));
      Add(TIDocField.Create('INSMK05'        ,   1));
      Add(TIDocField.Create('BWART05'        ,   3));
      Add(TIDocField.Create('GRUND05'        ,   4));
      Add(TIDocField.Create('INSMK06'        ,   1));
      Add(TIDocField.Create('BWART06'        ,   3));
      Add(TIDocField.Create('GRUND06'        ,   4));
      Add(TIDocField.Create('INSMK07'        ,   1));
      Add(TIDocField.Create('BWART07'        ,   3));
      Add(TIDocField.Create('GRUND07'        ,   4));
      Add(TIDocField.Create('INSMK08'        ,   1));
      Add(TIDocField.Create('BWART08'        ,   3));
      Add(TIDocField.Create('GRUND08'        ,   4));
      Add(TIDocField.Create('INSMK09'        ,   1));
      Add(TIDocField.Create('BWART09'        ,   3));
      Add(TIDocField.Create('GRUND09'        ,   4));
      Add(TIDocField.Create('INSMK10'        ,   1));
      Add(TIDocField.Create('BWART10'        ,   3));
      Add(TIDocField.Create('GRUND10'        ,   4));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL26001')) do begin
      Add(TIDocField.Create('PSTYV'          ,   4));
      Add(TIDocField.Create('MATKL'          ,   9));
      Add(TIDocField.Create('PROTH'          ,  18));
      Add(TIDocField.Create('UMVKZ'          ,   6));
      Add(TIDocField.Create('UMVKN'          ,   6));
      Add(TIDocField.Create('KZTLF'          ,   1));
      Add(TIDocField.Create('UEBTK'          ,   1));
      Add(TIDocField.Create('UEBTO'          ,   5));
      Add(TIDocField.Create('UNTTO'          ,   5));
      Add(TIDocField.Create('CHSPL'          ,   1));
      Add(TIDocField.Create('XCHBW'          ,   1));
      Add(TIDocField.Create('POSAR'          ,   1));
      Add(TIDocField.Create('SOBKZ'          ,   1));
      Add(TIDocField.Create('PCKPF'          ,   1));
      Add(TIDocField.Create('MAGRV'          ,   4));
      Add(TIDocField.Create('SHKZG'          ,   1));
      Add(TIDocField.Create('KOQUI'          ,   1));
      Add(TIDocField.Create('AKTNR'          ,  10));
      Add(TIDocField.Create('KZUMW'          ,   1));
      Add(TIDocField.Create('KVGR1'          ,   3));
      Add(TIDocField.Create('KVGR2'          ,   3));
      Add(TIDocField.Create('KVGR3'          ,   3));
      Add(TIDocField.Create('KVGR4'          ,   3));
      Add(TIDocField.Create('KVGR5'          ,   3));
      Add(TIDocField.Create('MVGR1'          ,   3));
      Add(TIDocField.Create('MVGR2'          ,   3));
      Add(TIDocField.Create('MVGR3'          ,   3));
      Add(TIDocField.Create('MVGR4'          ,   3));
      Add(TIDocField.Create('MVGR5'          ,   3));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL19')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('PARAM'          ,  20));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL43000')) do begin
      Add(TIDocField.Create('QUALF'          ,   1));
      Add(TIDocField.Create('BELNR'          ,  35));
      Add(TIDocField.Create('POSNR'          ,   6));
      Add(TIDocField.Create('DATUM'          ,   8));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL41000')) do begin
      Add(TIDocField.Create('QUALI'          ,   3));
      Add(TIDocField.Create('BSTNR'          ,  35));
      Add(TIDocField.Create('BSTDT'          ,   8));
      Add(TIDocField.Create('BSARK'          ,   4));
      Add(TIDocField.Create('IHREZ'          ,  12));
      Add(TIDocField.Create('POSEX'          ,   6));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL37003')) do begin
      Add(TIDocField.Create('EXIDV'          ,  20));
      Add(TIDocField.Create('TARAG'          ,  17));
      Add(TIDocField.Create('GWEIT'          ,   3));
      Add(TIDocField.Create('BRGEW'          ,  17));
      Add(TIDocField.Create('NTGEW'          ,  17));
      Add(TIDocField.Create('MAGEW'          ,  17));
      Add(TIDocField.Create('GWEIM'          ,   3));
      Add(TIDocField.Create('BTVOL'          ,  17));
      Add(TIDocField.Create('NTVOL'          ,  17));
      Add(TIDocField.Create('MAVOL'          ,  17));
      Add(TIDocField.Create('VOLEM'          ,   3));
      Add(TIDocField.Create('TAVOL'          ,  17));
      Add(TIDocField.Create('VOLET'          ,   3));
      Add(TIDocField.Create('VEGR2'          ,   5));
      Add(TIDocField.Create('VEGR1'          ,   5));
      Add(TIDocField.Create('VEGR3'          ,   5));
      Add(TIDocField.Create('VHILM'          ,  18));
      Add(TIDocField.Create('VEGR4'          ,   5));
      Add(TIDocField.Create('LAENG'          ,  15));
      Add(TIDocField.Create('VEGR5'          ,   5));
      Add(TIDocField.Create('BREIT'          ,  15));
      Add(TIDocField.Create('HOEHE'          ,  15));
      Add(TIDocField.Create('MEABM'          ,   3));
      Add(TIDocField.Create('INHALT'         ,  40));
      Add(TIDocField.Create('VHART'          ,   4));
      Add(TIDocField.Create('MAGRV'          ,   4));
      Add(TIDocField.Create('LADLG'          ,   8));
      Add(TIDocField.Create('LADEH'          ,   3));
      Add(TIDocField.Create('FARZT'          ,   4));
      Add(TIDocField.Create('FAREH'          ,   3));
      Add(TIDocField.Create('ENTFE'          ,   8));
      Add(TIDocField.Create('EHENT'          ,   3));
      Add(TIDocField.Create('VELTP'          ,   1));
      Add(TIDocField.Create('EXIDV2'         ,  20));
      Add(TIDocField.Create('LANDT'          ,   3));
      Add(TIDocField.Create('LANDF'          ,   3));
      Add(TIDocField.Create('NAMEF'          ,  35));
      Add(TIDocField.Create('NAMBE'          ,  35));
      Add(TIDocField.Create('VHILM_KU'       ,  22));
      Add(TIDocField.Create('VEBEZ'          ,  40));
      Add(TIDocField.Create('SMGKN'          ,   1));
      Add(TIDocField.Create('KDMAT35'        ,  35));
      Add(TIDocField.Create('SORTL'          ,  10));
      Add(TIDocField.Create('ERNAM'          ,  12));
      Add(TIDocField.Create('GEWFX'          ,   1));
      Add(TIDocField.Create('ERLKZ'          ,   1));
      Add(TIDocField.Create('EXIDA'          ,   1));
      Add(TIDocField.Create('MOVE_STATUS'    ,   4));
      Add(TIDocField.Create('PACKVORSCHR'    ,  22));
      Add(TIDocField.Create('PACKVORSCHR_ST' ,   1));
      Add(TIDocField.Create('LABELTYP'       ,   1));
      Add(TIDocField.Create('ZUL_AUFL'       ,  17));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL39000')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('PARAM'          ,  20));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL38')) do begin
      Add(TIDocField.Create('VEGR1_BEZ'      ,  20));
      Add(TIDocField.Create('VEGR2_BEZ'      ,  20));
      Add(TIDocField.Create('VEGR3_BEZ'      ,  20));
      Add(TIDocField.Create('VEGR4_BEZ'      ,  20));
      Add(TIDocField.Create('VEGR5_BEZ'      ,  20));
      Add(TIDocField.Create('VHART_BEZ'      ,  20));
      Add(TIDocField.Create('MAGRV_BEZ'      ,  20));
      Add(TIDocField.Create('VEBEZ'          ,  40));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL44002')) do begin
      Add(TIDocField.Create('VELIN'          ,   1));
      Add(TIDocField.Create('VBELN'          ,  10));
      Add(TIDocField.Create('POSNR'          ,   6));
      Add(TIDocField.Create('EXIDV'          ,  20));
      Add(TIDocField.Create('VEMNG'          ,  17));
      Add(TIDocField.Create('VEMEH'          ,   3));
      Add(TIDocField.Create('MATNR'          ,  18));
      Add(TIDocField.Create('KDMAT'          ,  35));
      Add(TIDocField.Create('CHARG'          ,  10));
      Add(TIDocField.Create('WERKS'          ,   4));
      Add(TIDocField.Create('LGORT'          ,   4));
      Add(TIDocField.Create('CUOBJ'          ,  18));
      Add(TIDocField.Create('BESTQ'          ,   1));
      Add(TIDocField.Create('SOBKZ'          ,   1));
      Add(TIDocField.Create('SONUM'          ,  16));
      Add(TIDocField.Create('ANZSN'          ,  11));
      Add(TIDocField.Create('WDATU'          ,   8));
      Add(TIDocField.Create('PARID'          ,  35));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDL21')) do begin
      Add(TIDocField.Create('LFART'          ,   4));
      Add(TIDocField.Create('BZIRK'          ,   6));
      Add(TIDocField.Create('AUTLF'          ,   1));
      Add(TIDocField.Create('EXPKZ'          ,   1));
      Add(TIDocField.Create('LIFSK'          ,   2));
      Add(TIDocField.Create('LPRIO'          ,   2));
      Add(TIDocField.Create('KDGRP'          ,   2));
      Add(TIDocField.Create('BEROT'          ,  20));
      Add(TIDocField.Create('TRAGR'          ,   4));
      Add(TIDocField.Create('TRSPG'          ,   2));
      Add(TIDocField.Create('AULWE'          ,  10));
   end;

   with IDoc.Add(TIDocRecord.Create('Z2Y11V_DELVRY_HEAD000')) do begin
      Add(TIDocField.Create('DISTRIB'        ,   1));
      Add(TIDocField.Create('MAXZAHL'        ,   2));
      Add(TIDocField.Create('FSTOFF'         ,   2));
      Add(TIDocField.Create('KUPACK'         ,   2));
      Add(TIDocField.Create('PSIL'           ,   1));
      Add(TIDocField.Create('NUM_PS'         ,   3));
      Add(TIDocField.Create('PSILL'          ,   4));
      Add(TIDocField.Create('LBLS'           ,   1));
      Add(TIDocField.Create('NUM_LB'         ,   3));
      Add(TIDocField.Create('LBLSL'          ,   4));
      Add(TIDocField.Create('LBLSMB'         ,   1));
      Add(TIDocField.Create('NUM_LM'         ,   3));
      Add(TIDocField.Create('LBLSML'         ,   4));
      Add(TIDocField.Create('PRICE'          ,   2));
      Add(TIDocField.Create('SPLIT'          ,   1));
      Add(TIDocField.Create('SHPMT'          ,   1));
      Add(TIDocField.Create('P1_VON'         ,   3));
      Add(TIDocField.Create('FIXROUT'        ,   1));
      Add(TIDocField.Create('COD'            ,   1));
      Add(TIDocField.Create('ZTERM'          ,   4));
      Add(TIDocField.Create('P1_BIS'         ,   3));
      Add(TIDocField.Create('P1_HLF'         ,   3));
      Add(TIDocField.Create('P2_BIS'         ,   3));
      Add(TIDocField.Create('P2_VON'         ,   3));
      Add(TIDocField.Create('P2_HLF'         ,   3));
      Add(TIDocField.Create('P3_VON'         ,   3));
      Add(TIDocField.Create('P3_BIS'         ,   3));
      Add(TIDocField.Create('P3_HLF'         ,   3));
      Add(TIDocField.Create('P4_VON'         ,   3));
      Add(TIDocField.Create('P4_BIS'         ,   3));
      Add(TIDocField.Create('P4_HLF'         ,   3));
      Add(TIDocField.Create('V_CODE'         ,   3));
      Add(TIDocField.Create('WEIGHT'         ,  15));
      Add(TIDocField.Create('MEINS'          ,   3));
      Add(TIDocField.Create('VLABEL'         ,   3));
      Add(TIDocField.Create('CANCEL_'        ,   1));
      Add(TIDocField.Create('DELDAT'         ,   8));
      Add(TIDocField.Create('DELTIM'         ,   6));
      Add(TIDocField.Create('PICKDIF'        ,   1));
      Add(TIDocField.Create('YV2_KOPRI'      ,   2));
      Add(TIDocField.Create('STCEG'          ,  20));
      Add(TIDocField.Create('TDIIX'          ,   1));
   end;

   with IDoc.Add(TIDocRecord.Create('Z2Y11V_DELVRY_ITEM000')) do begin
      Add(TIDocField.Create('KSL1'           ,   2));
      Add(TIDocField.Create('KSL2'           ,   2));
      Add(TIDocField.Create('KSL3'           ,   2));
      Add(TIDocField.Create('KSL4'           ,   2));
      Add(TIDocField.Create('KSL5'           ,   2));
      Add(TIDocField.Create('PACK_V'         ,   3));
      Add(TIDocField.Create('PACK_B'         ,   3));
      Add(TIDocField.Create('LANG'           ,   3));
      Add(TIDocField.Create('BREIT'          ,   3));
      Add(TIDocField.Create('HOCH'           ,   3));
      Add(TIDocField.Create('STUECK'         ,  17));
      Add(TIDocField.Create('AUART'          ,   4));
      Add(TIDocField.Create('BEZEI'          ,  25));
      Add(TIDocField.Create('KWMENG'         ,  17));
      Add(TIDocField.Create('CODGE'          ,  15));
      Add(TIDocField.Create('AST'            ,   7));
      Add(TIDocField.Create('FIRST_DELIV'    ,   1));
      Add(TIDocField.Create('OLMNG'          ,  15));
      Add(TIDocField.Create('PRINT_KZ'       ,   1));
   end;

   with IDoc.Add(TIDocRecord.Create('E2MBXYH001')) do begin
      Add(TIDocField.Create('BLDAT'          ,   8, ftDate));
      Add(TIDocField.Create('BUDAT'          ,   8, ftDate));
      Add(TIDocField.Create('XBLNR'          ,  16));
      Add(TIDocField.Create('BKTXT'          ,  25));
      Add(TIDocField.Create('FRBNR'          ,  16));
      Add(TIDocField.Create('XABLN'          ,  10));
      Add(TIDocField.Create('TCODE'          ,  20));
      Add(TIDocField.Create('USNAM'          ,  12));
      Add(TIDocField.Create('VBUND'          ,   6));
   end;

   with IDoc.Add(TIDocRecord.Create('E2MBXYI')) do begin
      Add(TIDocField.Create('BEAKZ'          ,   1));
      Add(TIDocField.Create('XSTOB'          ,   1));
      Add(TIDocField.Create('MATNR'          ,  18));
      Add(TIDocField.Create('WERKS'          ,   4));
      Add(TIDocField.Create('LGORT'          ,   4));
      Add(TIDocField.Create('CHARG'          ,  10));
      Add(TIDocField.Create('BWART'          ,   3));
      Add(TIDocField.Create('INSMK'          ,   1));
      Add(TIDocField.Create('SOBKZ'          ,   1));
      Add(TIDocField.Create('KZVBR'          ,   1));
      Add(TIDocField.Create('LIFNR'          ,  10));
      Add(TIDocField.Create('KUNNR'          ,  10));
      Add(TIDocField.Create('KDAUF'          ,  10));
      Add(TIDocField.Create('KDPOS'          ,   6));
      Add(TIDocField.Create('KDEIN'          ,   4));
      Add(TIDocField.Create('SHKZG'          ,   1));
      Add(TIDocField.Create('WAERS'          ,   5));
      Add(TIDocField.Create('DMBTR'          ,  15));
      Add(TIDocField.Create('BWTAR'          ,  10));
      Add(TIDocField.Create('ERFMG'          ,  15));
      Add(TIDocField.Create('ERFME'          ,   3));
      Add(TIDocField.Create('BPMNG'          ,  15));
      Add(TIDocField.Create('BPRME'          ,   3));
      Add(TIDocField.Create('EBELN'          ,  10));
      Add(TIDocField.Create('EBELP'          ,   5));
      Add(TIDocField.Create('ELIKZ'          ,   1));
      Add(TIDocField.Create('SGTXT'          ,  50));
      Add(TIDocField.Create('WEMPF'          ,  12));
      Add(TIDocField.Create('ABLAD'          ,  25));
      Add(TIDocField.Create('KOSTL'          ,  10));
      Add(TIDocField.Create('AUFNR'          ,  12));
      Add(TIDocField.Create('ANLN1'          ,  12));
      Add(TIDocField.Create('ANLN2'          ,   4));
      Add(TIDocField.Create('RSNUM'          ,  10));
      Add(TIDocField.Create('RSPOS'          ,   4));
      Add(TIDocField.Create('KZEAR'          ,   1));
      Add(TIDocField.Create('UMMAT'          ,  18));
      Add(TIDocField.Create('UMWRK'          ,   4));
      Add(TIDocField.Create('UMLGO'          ,   4));
      Add(TIDocField.Create('UMCHA'          ,  10));
      Add(TIDocField.Create('KZBEW'          ,   1));
      Add(TIDocField.Create('WEUNB'          ,   1));
      Add(TIDocField.Create('LGNUM'          ,   3));
      Add(TIDocField.Create('LGTYP'          ,   3));
      Add(TIDocField.Create('LGPLA'          ,  10));
      Add(TIDocField.Create('GRUND'          ,   4));
      Add(TIDocField.Create('EVERS'          ,   2));
      Add(TIDocField.Create('EVERE'          ,   2));
      Add(TIDocField.Create('IMKEY'          ,   8));
      Add(TIDocField.Create('KSTRG'          ,  12));
      Add(TIDocField.Create('PAOBJNR'        ,  10));
      Add(TIDocField.Create('PRCTR'          ,  10));
      Add(TIDocField.Create('PS_PSP_PNR'     ,   8));
      Add(TIDocField.Create('NPLNR'          ,  12));
      Add(TIDocField.Create('AUFPL'          ,  10));
      Add(TIDocField.Create('APLZL'          ,   8));
      Add(TIDocField.Create('AUFPS'          ,   4));
      Add(TIDocField.Create('VPTNR'          ,  10));
      Add(TIDocField.Create('FIPOS'          ,  14));
      Add(TIDocField.Create('GSBER'          ,   4));
      Add(TIDocField.Create('BSTMG'          ,  15));
      Add(TIDocField.Create('BSTME'          ,   3));
      Add(TIDocField.Create('EXBWR'          ,  15));
      Add(TIDocField.Create('KONTO'          ,  10));
      Add(TIDocField.Create('RSHKZ'          ,   1));
      Add(TIDocField.Create('BDMNG'          ,  15));
      Add(TIDocField.Create('ENMNG'          ,  15));
      Add(TIDocField.Create('QPLOS'          ,  12));
      Add(TIDocField.Create('UMZST'          ,   1));
      Add(TIDocField.Create('UMZUS'          ,   1));
      Add(TIDocField.Create('UMBAR'          ,  10));
      Add(TIDocField.Create('UMSOK'          ,   1));
      Add(TIDocField.Create('LFBJA'          ,   4));
      Add(TIDocField.Create('LFBNR'          ,  10));
      Add(TIDocField.Create('LFPOS'          ,   4));
      Add(TIDocField.Create('SJAHR'          ,   4));
      Add(TIDocField.Create('SMBLN'          ,  10));
      Add(TIDocField.Create('SMBLP'          ,   4));
      Add(TIDocField.Create('EXVKW'          ,  15));
      Add(TIDocField.Create('QM_ZUSTD'       ,   1));
      Add(TIDocField.Create('POSNR'          ,   6));
      Add(TIDocField.Create('VBELN'          ,  10));
      Add(TIDocField.Create('QM_UMZST'       ,   1));
      Add(TIDocField.Create('BWLVS'          ,   3));
      Add(TIDocField.Create('UMREZ'          ,   5));
      Add(TIDocField.Create('UMREN'          ,   5));
      Add(TIDocField.Create('VFDAT'          ,   8));
      Add(TIDocField.Create('DABRZ'          ,   8));
   end;

   with IDoc.Add(TIDocRecord.Create('E2MBXYJ000')) do begin
      Add(TIDocField.Create('PARGB'          ,   4));
      Add(TIDocField.Create('PARBU'          ,   4));
      Add(TIDocField.Create('CLASS'          ,  18));
      Add(TIDocField.Create('UMCLA'          ,  18));
      Add(TIDocField.Create('XCLAS'          ,   1));
      Add(TIDocField.Create('UMXCL'          ,   1));
      Add(TIDocField.Create('XNIBU'          ,   1));
      Add(TIDocField.Create('BDTER'          ,   8));
      Add(TIDocField.Create('TBBEL'          ,  10));
      Add(TIDocField.Create('TBBPO'          ,   4));
      Add(TIDocField.Create('TBBJR'          ,   4));
      Add(TIDocField.Create('OBJNR'          ,  22));
      Add(TIDocField.Create('AUTYP'          ,   2));
      Add(TIDocField.Create('QPLOA'          ,  12));
      Add(TIDocField.Create('TBPKZ'          ,   1));
      Add(TIDocField.Create('TAFKZ'          ,   1));
      Add(TIDocField.Create('KZEAR_OLD'      ,   1));
      Add(TIDocField.Create('RSART'          ,   1));
      Add(TIDocField.Create('PPRCTR'         ,  10));
      Add(TIDocField.Create('XMEVO'          ,   1));
      Add(TIDocField.Create('UMLGT'          ,   3));
      Add(TIDocField.Create('UMLGP'          ,  10));
      Add(TIDocField.Create('MENGE'          ,  15));
      Add(TIDocField.Create('MEINS'          ,   3));
      Add(TIDocField.Create('FKBER'          ,   4));
      Add(TIDocField.Create('MHDAT'          ,   8));
      Add(TIDocField.Create('BSSKZ'          ,   1));
      Add(TIDocField.Create('EXIDV'          ,  20));
      Add(TIDocField.Create('BERKZ'          ,   1));
      Add(TIDocField.Create('PRVBE'          ,  10));
      Add(TIDocField.Create('KZECH'          ,   1));
      Add(TIDocField.Create('UPTYP'          ,   1));
      Add(TIDocField.Create('REFIX'          ,  11));
      Add(TIDocField.Create('VLIEF_AVIS'     ,  10));
      Add(TIDocField.Create('VBELP_AVIS'     ,   6));
      Add(TIDocField.Create('XWAIT'          ,   1));
      Add(TIDocField.Create('XNOEQ'          ,   1));
      Add(TIDocField.Create('ILINR'          ,   6));
      Add(TIDocField.Create('VOLUM'          ,  17));
      Add(TIDocField.Create('VOLEH'          ,   3));
      Add(TIDocField.Create('ANZL1'          ,   4));
      Add(TIDocField.Create('ANZL2'          ,   4));
      Add(TIDocField.Create('LMEN1'          ,  15));
      Add(TIDocField.Create('LMEN2'          ,  15));
      Add(TIDocField.Create('LETY1'          ,   3));
      Add(TIDocField.Create('LETY2'          ,   3));
      Add(TIDocField.Create('KZKUB'          ,   1));
      Add(TIDocField.Create('UBTYP'          ,   3));
      Add(TIDocField.Create('UBLGP'          ,  10));
      Add(TIDocField.Create('MBLNR'          ,  10));
      Add(TIDocField.Create('MBLPO'          ,   4));
      Add(TIDocField.Create('MJAHR'          ,   4));
      Add(TIDocField.Create('URZEI'          ,   4));
      Add(TIDocField.Create('GEBER'          ,  10));
      Add(TIDocField.Create('FISTL'          ,  16));
      Add(TIDocField.Create('KZBWS'          ,   1));
      Add(TIDocField.Create('KDAUF_SD'       ,  10));
      Add(TIDocField.Create('KDPOS_SD'       ,   6));
      Add(TIDocField.Create('XRERE'          ,   1));
      Add(TIDocField.Create('XSTOR'          ,   1));
      Add(TIDocField.Create('BRGEW'          ,  17));
      Add(TIDocField.Create('GEWEI'          ,   3));
      Add(TIDocField.Create('WM_KZBEW'       ,   1));
      Add(TIDocField.Create('WENUM'          ,  10));
      Add(TIDocField.Create('GEBEH'          ,   3));
      Add(TIDocField.Create('ANZGEB'         ,   9));
      Add(TIDocField.Create('KANBA'          ,   1));
      Add(TIDocField.Create('BSTMG'          ,  15));
   end;

   with IDoc.Add(TIDocRecord.Create('E2MARAM003')) do begin
      Add(TIDocField.Create('MSGFN'          ,   3));
      Add(TIDocField.Create('MATNR'          ,  18));
      Add(TIDocField.Create('ERSDA'          ,   8));
      Add(TIDocField.Create('ERNAM'          ,  12));
      Add(TIDocField.Create('LAEDA'          ,   8));
      Add(TIDocField.Create('AENAM'          ,  12));
      Add(TIDocField.Create('PSTAT'          ,  15));
      Add(TIDocField.Create('LVORM'          ,   1));
      Add(TIDocField.Create('MTART'          ,   4));
      Add(TIDocField.Create('MBRSH'          ,   1));
      Add(TIDocField.Create('MATKL'          ,   9));
      Add(TIDocField.Create('BISMT'          ,  18));
      Add(TIDocField.Create('MEINS'          ,   3));
      Add(TIDocField.Create('BSTME'          ,   3));
      Add(TIDocField.Create('ZEINR'          ,  22));
      Add(TIDocField.Create('ZEIAR'          ,   3));
      Add(TIDocField.Create('ZEIVR'          ,   2));
      Add(TIDocField.Create('ZEIFO'          ,   4));
      Add(TIDocField.Create('AESZN'          ,   6));
      Add(TIDocField.Create('BLATT'          ,   3));
      Add(TIDocField.Create('BLANZ'          ,   3));
      Add(TIDocField.Create('FERTH'          ,  18));
      Add(TIDocField.Create('FORMT'          ,   4));
      Add(TIDocField.Create('GROES'          ,  32));
      Add(TIDocField.Create('WRKST'          ,  14));
      Add(TIDocField.Create('NORMT'          ,  18));
      Add(TIDocField.Create('LABOR'          ,   3));
      Add(TIDocField.Create('EKWSL'          ,   4));
      Add(TIDocField.Create('BRGEW'          ,  14));
      Add(TIDocField.Create('NTGEW'          ,  14));
      Add(TIDocField.Create('GEWEI'          ,   3));
      Add(TIDocField.Create('VOLUM'          ,  14));
      Add(TIDocField.Create('VOLEH'          ,   3));
      Add(TIDocField.Create('BEHVO'          ,   2));
      Add(TIDocField.Create('RAUBE'          ,   2));
      Add(TIDocField.Create('TEMPB'          ,   2));
      Add(TIDocField.Create('TRAGR'          ,   4));
      Add(TIDocField.Create('STOFF'          ,  18));
      Add(TIDocField.Create('SPART'          ,   2));
      Add(TIDocField.Create('KUNNR'          ,  10));
      Add(TIDocField.Create('WESCH'          ,  14));
      Add(TIDocField.Create('BWVOR'          ,   1));
      Add(TIDocField.Create('BWSCL'          ,   1));
      Add(TIDocField.Create('SAISO'          ,   4));
      Add(TIDocField.Create('ETIAR'          ,   2));
      Add(TIDocField.Create('ETIFO'          ,   2));
      Add(TIDocField.Create('EAN11'          ,  18));
      Add(TIDocField.Create('NUMTP'          ,   2));
      Add(TIDocField.Create('LAENG'          ,  14));
      Add(TIDocField.Create('BREIT'          ,  14));
      Add(TIDocField.Create('HOEHE'          ,  14));
      Add(TIDocField.Create('MEABM'          ,   3));
      Add(TIDocField.Create('PRDHA'          ,  18));
      Add(TIDocField.Create('CADKZ'          ,   1));
      Add(TIDocField.Create('ERGEW'          ,  14));
      Add(TIDocField.Create('ERGEI'          ,   3));
      Add(TIDocField.Create('ERVOL'          ,  14));
      Add(TIDocField.Create('ERVOE'          ,   3));
      Add(TIDocField.Create('GEWTO'          ,   3));
      Add(TIDocField.Create('VOLTO'          ,   3));
      Add(TIDocField.Create('VABME'          ,   1));
      Add(TIDocField.Create('LZKFG'          ,   1));
      Add(TIDocField.Create('XCHPF'          ,   1));
      Add(TIDocField.Create('VHART'          ,   4));
      Add(TIDocField.Create('FUELG'          ,   3));
      Add(TIDocField.Create('STFAK'          ,   5));
      Add(TIDocField.Create('MAGRV'          ,   4));
      Add(TIDocField.Create('BEGRU'          ,   4));
      Add(TIDocField.Create('QMPUR'          ,   1));
      Add(TIDocField.Create('RBNRM'          ,   9));
      Add(TIDocField.Create('MHDRZ'          ,   6));
      Add(TIDocField.Create('MHDHB'          ,   6));
      Add(TIDocField.Create('MHDLP'          ,   5));
      Add(TIDocField.Create('VPSTA'          ,  15));
      Add(TIDocField.Create('EXTWG'          ,  18));
      Add(TIDocField.Create('MSTAE'          ,   2));
      Add(TIDocField.Create('MSTAV'          ,   2));
      Add(TIDocField.Create('MSTDE'          ,   8));
      Add(TIDocField.Create('MSTDV'          ,   8));
      Add(TIDocField.Create('KZUMW'          ,   1));
      Add(TIDocField.Create('KOSCH'          ,  18));
      Add(TIDocField.Create('NRFHG'          ,   1));
      Add(TIDocField.Create('MFRPN'          ,  40));
      Add(TIDocField.Create('MFRNR'          ,  10));
      Add(TIDocField.Create('BMATN'          ,  18));
      Add(TIDocField.Create('MPROF'          ,   4));
      Add(TIDocField.Create('PROFL'          ,   3));
      Add(TIDocField.Create('IHIVI'          ,   1));
      Add(TIDocField.Create('ILOOS'          ,   1));
      Add(TIDocField.Create('KZGVH'          ,   1));
      Add(TIDocField.Create('XGCHP'          ,   1));
      Add(TIDocField.Create('COMPL'          ,   2));
      Add(TIDocField.Create('KZEFF'          ,   1));
      Add(TIDocField.Create('RDMHD'          ,   1));
      Add(TIDocField.Create('IPRKZ'          ,   1));
      Add(TIDocField.Create('PRZUS'          ,   1));
      Add(TIDocField.Create('MTPOS_MARA'     ,   4));
      Add(TIDocField.Create('GEWTO_NEW'      ,   5));
      Add(TIDocField.Create('VOLTO_NEW'      ,   5));
   end;

   with IDoc.Add(TIDocRecord.Create('E2MAKTM001')) do begin
      Add(TIDocField.Create('MSGFN'          ,   3));
      Add(TIDocField.Create('SPRAS'          ,   1));
      Add(TIDocField.Create('MAKTX'          ,  40));
      Add(TIDocField.Create('SPRAS_ISO'      ,   2));
   end;

   with IDoc.Add(TIDocRecord.Create('E2ADHDR001')) do begin
      Add(TIDocField.Create('MESTYP'         ,   6));
      Add(TIDocField.Create('MESCOD'         ,   3));
      Add(TIDocField.Create('MESFCT'         ,   3));
      Add(TIDocField.Create('MESTYP_LNG'     ,  30));
   end;

   with IDoc.Add(TIDocRecord.Create('E2STATE002')) do begin
      Add(TIDocField.Create('DOCNUM'         ,  16));
      Add(TIDocField.Create('STATUS'         ,   2));
      Add(TIDocField.Create('STACOD'         ,   8));
      Add(TIDocField.Create('STATXT'         ,  70));
      Add(TIDocField.Create('STAPA1'         ,  20));
      Add(TIDocField.Create('STAPA2'         ,  20));
      Add(TIDocField.Create('STAPA3'         ,  20));
      Add(TIDocField.Create('STAPA4'         ,  20));
      Add(TIDocField.Create('STATYP'         ,   1));
      Add(TIDocField.Create('STAMQU'         ,   3));
      Add(TIDocField.Create('STAMID'         ,  20));
      Add(TIDocField.Create('STAMNO'         ,   3));
      Add(TIDocField.Create('STAPA1_LNG'     ,  50));
      Add(TIDocField.Create('STAPA2_LNG'     ,  50));
      Add(TIDocField.Create('STAPA3_LNG'     ,  50));
      Add(TIDocField.Create('STAPA4_LNG'     ,  50));
   end;

   with IDoc.Add(TIDocRecord.Create('E2PRTOB')) do begin
      Add(TIDocField.Create('DOCNUM'         ,  16));
      Add(TIDocField.Create('LOGSYS'         ,  10));
      Add(TIDocField.Create('OBJTYPE'        ,  10));
      Add(TIDocField.Create('OBJKEY'         ,  70));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDK01')) do begin
      Add(TIDocField.Create('FILLER01'       ,  79));
      Add(TIDocField.Create('BSART'          ,   4));
      Add(TIDocField.Create('BELNR'          ,  35));
      Add(TIDocField.Create('FILLER01'       ,  13));
      Add(TIDocField.Create('RECIPNT_NO'     ,  10));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDK14')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('ORGID'          ,  35));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDK03')) do begin
      Add(TIDocField.Create('IDDAT'          ,   3));
      Add(TIDocField.Create('DATUM'          ,   8));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDKA1')) do begin
      Add(TIDocField.Create('PARVW'          ,   3));
      Add(TIDocField.Create('PARTN'          ,  17));
      Add(TIDocField.Create('LFINR'          ,  17));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDK02')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('BELNR'          ,  35));
      Add(TIDocField.Create('FILLER01'       ,   6));
      Add(TIDocField.Create('DATUM'          ,   8));
      Add(TIDocField.Create('UZEIT'          ,   6));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDK17')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('LKOND'          ,   3));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDP01')) do begin
      Add(TIDocField.Create('POSEX'          ,   6));
      Add(TIDocField.Create('FILLER01'       ,   3));
      Add(TIDocField.Create('PSTYP'          ,   1));
      Add(TIDocField.Create('FILLER02'       , 377));
      Add(TIDocField.create('VSTEL'          ,   4));
      Add(TIDocField.Create('FILLER03'       ,   3));
      Add(TIDocField.Create('MATNR'          ,  35));
      Add(TIDocField.Create('FILLER04'       ,   8));
      Add(TIDocField.Create('HIEWV'          ,   1));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDP20')) do begin
      Add(TIDocField.Create('WMENG'          ,  15));
      Add(TIDocField.Create('AMENG'          ,  15));
      Add(TIDocField.Create('EDATU'          ,   8));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDP19')) do begin
      Add(TIDocField.Create('QUALF'          ,   3));
      Add(TIDocField.Create('IDTNR'          ,  35));
      Add(TIDocField.Create('KTEXT'          ,  70));
   end;

   with IDoc.Add(TIDocRecord.Create('ZE2SC01')) do begin
      Add(TIDocField.Create('ZCOMP'          ,  18));
      Add(TIDocField.Create('ZMENG'          ,  15));
      Add(TIDocField.Create('FILLER01'       ,   2));
      Add(TIDocField.Create('ZMENE'          ,   3));
   end;

   with IDoc.Add(TIDocRecord.Create('E2EDPS01')) do begin
      Add(TIDocField.Create('SUMID'          ,   3));
      Add(TIDocField.Create('SUMME'          ,  18));
      Add(TIDocField.Create('SUNIT'          ,   3));
   end;

end;

//------------------------------------------------------------------------------
function TfrmMain.InitSapDefsFromFile : boolean;
//------------------------------------------------------------------------------
// Read IDoc Definitions from PAR files, configuration from ini:
// [Files]
// ParFile1=<1st file name>
// ParFile2=<2nd file name>
// ...
// ParFilen=<nth file name>
//------------------------------------------------------------------------------
var
   Ini : TIniFile;
   Parser : TSapParse;
   i : integer;
   FileList : TStringList;
   FileName : string;
begin
   result := false;
   Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
   FileList := TStringList.Create;
   try
      i := 0;
      while i < maxint do
      begin
         inc(i);
         FileName := Ini.ReadString('Files', Format('ParFile%d', [i]), '');
         if (FileName <> '') then
         begin
            if not FileExists(FileName) then
               // Map filenames to relative directory of application.
               // So app can be started from any directory and will
               // still find the par files
               FileName := ExtractFilePath(Application.ExeName)+FileName;
            if FileExists(FileName) then
               FileList.Add(FileName);
         end
         else
            break;
      end;
      if FileList.Count > 0 then
      begin
         ProgressFileRead.Properties.Max := FileList.Count;
         ProgressFileRead.Position := 0;
         Parser := TSapParse.Create;
         try
            for i := 0 to FileList.Count-1 do
            begin
               Parser.ReadFromFile(FileList[i], IDoc);
               ProgressFileRead.Position := i+1;
               Application.ProcessMessages;
            end;
            StatusBar.Panels[1].Visible := true;
            lblParFilesInfo.Caption := Format('%d PAR', [FileList.Count]);
            lblParFilesInfo.Hint :=  'Using definitions from files:'#13#13+FileList.Text;
            result := true;
         finally
            Parser.Free;
            ProgressFileRead.Hide;
         end;
      end;
   finally
      FileList.Free;
      Ini.Free
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.TreeListResize(Sender: TObject);
//------------------------------------------------------------------------------
// Adjust the Treelist if window is resized
//------------------------------------------------------------------------------
begin
   ColumnNames.Width := 91 * Trunc(TWinControl(Sender).Width/228);
   ColumnData.Width := TWinControl(Sender).ClientWidth - ColumnNames.Width-4;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.TreeListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//------------------------------------------------------------------------------
// Key handler for the treeview
//------------------------------------------------------------------------------

   //------------------------------------------------------------------------------
   function FindStringInTreeView(Sender : TcxTreeList; Value : string; SearchForward : boolean = true) : boolean;
   //------------------------------------------------------------------------------
   // Inline function for finding a string within the treeview
   //------------------------------------------------------------------------------
   var
      FoundNode, ParentNode : TcxTreeListNode;
   begin
      result := false;
      with Sender do
      begin
         FoundNode := TreeList.FindNodeByText(Value,Columns[1], FocusedNode, false, SearchForward);

         if FoundNode <> nil then
         begin
            result := true;
            ParentNode := FoundNode;
            while ParentNode.Parent <> nil do
            begin
               ParentNode := ParentNode.Parent;
               ParentNode.Expanded := true;
            end;
            FocusedNode := FoundNode;
         end;
      end;
   end;

var
   sFilter : string;

begin
   if (ssCtrl in Shift) then
   begin
      case Key of
      ord('F') :  // Enter search string
         begin
            Key := 0;
            sFilter := FilterValue;
            if InputQuery('Search', 'Search for:', sFilter) then
            begin
               FilterValue := sFilter;
               FindStringInTreeView(TcxTreeList(Sender), sFilter);
            end;
         end;
      ord('S') : // Save IDoc to file
         begin
            SaveToFile(TcxTreeList(Sender));
         end;
      VK_F3 : // find previous
         begin
            if FilterValue <> '' then
            begin
               TcxTreeList(Sender).GotoPrev;
               if not FindStringInTreeView(TcxTreeList(Sender), FilterValue, false) then
                  TcxTreeList(Sender).GotoNext;

            end;
         end;
      VK_DELETE :  // Delete current node
         begin
            Key := 0;
            TreeList.BeginUpdate;
            TreeList.FocusedNode.Delete;
            TreeList.EndUpdate;
         end;
      VK_UP : // Sibling up
         with TcxTreeList(Sender) do
         begin
            if FocusedNode.HasChildren then
               if FocusedNode.GetPrevSibling <> nil then
                  FocusedNode.GetPrevSibling.Focused := true
            else
               FocusedNode.Parent.Focused := true;
            Key := 0;
         end;
      VK_DOWN : // Sibling down
         with TcxTreeList(Sender) do
         begin
            if FocusedNode.HasChildren then
               if FocusedNode.getNextSibling <> nil then
                  FocusedNode.getNextSibling.Focused := true
            else
               FocusedNode.Parent.getNextSibling;
            Key := 0;
         end;
      end;
   end
   else
   begin
      case Key of
      VK_F3 :  // Find next
         begin
            if FilterValue <> '' then
            begin
               TcxTreeList(Sender).GotoNext;
               if not FindStringInTreeView(TcxTreeList(Sender), FilterValue) then
                  TcxTreeList(Sender).GotoPrev;

            end;
         end;
      VK_LEFT : // Level up
         with TcxTreeList(Sender) do
         begin
            if FocusedNode.HasChildren then
               FocusedNode.Expanded := false
            else
               if not IsEditing and (FocusedNode.Parent <> nil) then
                  FocusedNode.Parent.Focused := true;
         end;
      VK_RIGHT : // Level down
         with TcxTreeList(Sender) do
         begin
            if FocusedNode.HasChildren then
            begin
               FocusedNode.Expanded := true;
            end;
         end;
      end;
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.WMDROPFILES(var Msg: TMessage);
//------------------------------------------------------------------------------
// Handler for file drag and drop
//------------------------------------------------------------------------------
var
  pcFileName: PChar;
  i, iSize, iFileCount: integer;
begin
  pcFileName := ''; // to avoid compiler warning message
  iFileCount := DragQueryFile(Msg.wParam, $FFFFFFFF, pcFileName, 255);
  for i := 0 to iFileCount - 1 do
  begin
    iSize := DragQueryFile(Msg.wParam, i, nil, 0) + 1;
    pcFileName := StrAlloc(iSize);
    DragQueryFile(Msg.wParam, i, pcFileName, iSize);
    if FileExists(pcFileName) then
    begin
      EditFileName.Text := pcFileName;
      btnReadClick(btnRead);
    end;
    StrDispose(pcFileName);
  end;
  DragFinish(Msg.wParam);
end;

//------------------------------------------------------------------------------
procedure TfrmMain.LBWindowProc(var Message: TMessage);
//------------------------------------------------------------------------------
// Message handler for drag and drop message
//------------------------------------------------------------------------------
begin
  if Message.Msg = WM_DROPFILES then
    WMDROPFILES(Message); // handle WM_DROPFILES message
  OldLBWindowProc(Message);
end;

//------------------------------------------------------------------------------
procedure TfrmMain.EMailDblClick(Sender: TObject);
//------------------------------------------------------------------------------
// Send an eMail to the author
//------------------------------------------------------------------------------
begin
   ShellExecute(handle, 'open', PChar('mailto:info@softwareunion.lu?subject=SDV Data Viewer'), nil, nil, SW_SHOWNORMAL);
end;

//------------------------------------------------------------------------------
procedure TfrmMain.CheckFileParams(const sWildcard : string);
//------------------------------------------------------------------------------
// Process wildcards
//------------------------------------------------------------------------------
var
   sr : TSearchRec;
   sDir : string;
begin
   if FindFirst(sWildCard, faArchive, sr) = 0 then
   begin
      sDir := ExtractFilePath(sWildcard);
      repeat
         EditFileName.Text := sDir+sr.Name;
         btnRead.Click;
      until FindNext(sr) <> 0;
      FindClose(sr);
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.SaveToFile(Sender : TcxTreeList);
//------------------------------------------------------------------------------
// Save the current IDoc to file
//------------------------------------------------------------------------------
var
   ANode, AChild : TcxTreelistNode;
   FileName : string;
   OutputIDoc : TIDoc;
   SegmentName : string;
begin
   if Sender.Count < 1 then
      exit;

   // Get first node
   ANode := Sender.FocusedNode;
   while ANode.Parent <> TreeList.Nodes.Root do
   begin
      ANode := ANode.Parent;
   end;

   // Get filename
   FileName := ANode.Values[0];

   if not (MessageDlg('Save data to '+FileName+#13+'(Existing file will be overwritten)', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then exit;

   // Create IDoc
   OutputIDoc := TIDoc.Create;
   try
      OutPutIDoc.Delimiter := ANode.Values[1];
      if OutPutIDoc.Delimiter <> '' then
         OutPutIdoc.AppendCrLf := true;

      ANode := ANode.GetFirstChild;

      while ANode <> nil do
      begin
         // Create the segment
         SegmentName := trim(ANode.Values[0]);
         if pos(' ', SegmentName) > 0 then
            SegmentName := copy(SegmentName,1,pos(' ', SegmentName)-1);

         with OutputIDoc.Add(TIDocRecord.Create(SegmentName, false)) do
         begin
            Assign(IDoc.SegmentByName(SegmentName));
            AChild := ANode.GetFirstChild;
            while AChild <> nil do
            begin
               // Assign the field values
               try
                  with FieldByName(AChild.Values[0]) do
                  begin
                     if AChild.Values[1] = Null then
                        Data := ''
                     else
                     begin
                        case FieldType of
                        ftString  : Data := AChild.Values[1];
                        ftDate    : AsDateTime := StrToDate(AChild.Values[1]);
                        ftTime    : AsTime     := StrToTime(AChild.Values[1]);
                        ftInteger : AsInteger  := StrToIntDef(AChild.Values[1],0);
                        ftFloat   : AsFloat    := StrToFloatDef(AChild.Values[1],0);
                        else
                           Data := AChild.Values[1];
                        end;
                     end;
                  end;
               except
                  on e : Exception do Showmessage(e.Message);
               end;
               AChild := AChild.GetNextSibling;
            end;
         end;

         ANode := ANode.GetNextChild(ANode);

         // Don't process next file
         if ANode <> nil then
            if ANode.Parent = TreeList.Nodes.Root then
               ANode := nil;
      end;
      // Save IDoc to file - original will be overwritten
      OutputIDoc.OnSaveToFile := IDocFileProgress;
      ProgressFileRead.Show;
      ProgressFileRead.Properties.EndColor := clPurple;
      OutputIDoc.SaveToFile(FileName);
   finally
      ProgressFileRead.Hide;
      OutputIDoc.Free;
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.TreeListCustomDrawCell(Sender: TObject;
  ACanvas: TcxCanvas; AViewInfo: TcxTreeListEditCellViewInfo;
  var ADone: Boolean);
//------------------------------------------------------------------------------
// Show header records in BOLD
//------------------------------------------------------------------------------
begin
   if copy(AViewInfo.Node.Values[0],1,8) = 'EDI_DC40' then
   begin
      ACanvas.Font.Style := [fsBold];
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.ColumnDataGetEditProperties(
  Sender: TcxTreeListColumn; ANode: TcxTreeListNode;
  var EditProperties: TcxCustomEditProperties);
//------------------------------------------------------------------------------
// Decide what editing style to be taken depending on IDoc-Field Definition
//------------------------------------------------------------------------------
var
   IDocField : TIDocField;
begin
   if Sender.Name = 'ColumnData' then
   begin
      if ANode.ValueCount = 3 then
      begin
         if VarIsStr(ANode.Values[2]) and
            VarIsStr(ANode.Values[0]) then
         begin
            IDocField := IDoc.SegmentByName(ANode.Values[2]).FieldByName(ANode.Values[0]);
            case IDocField.FieldType of
            ftDate : EditProperties := DateEditProperties;
            ftTime : EditProperties := TimeEditProperties;
            ftFloat, ftInteger :
                  begin
                     EditProperties := NumberEditProperties;
                     TcxCurrencyEditProperties(EditProperties).DecimalPlaces := IDocField.Decimals;
                     if IDocField.Decimals = 0 then
                        TcxCurrencyEditProperties(EditProperties).DisplayFormat := '0'
                     else
                        TcxCurrencyEditProperties(EditProperties).DisplayFormat := '0.'+StringOfChar('0', IDocField.Decimals);
                  end;
            else
               begin
                  EditProperties := StringEditProperties;
                  TcxTextEditProperties(EditProperties).MaxLength := IDocField.Size;
               end;
            end;
         end
      end;
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.EditFileNameKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//------------------------------------------------------------------------------
// For those without a mouse to click the ellipsis button
//------------------------------------------------------------------------------
begin
   if (Shift = []) and (Key = VK_F4) then
   begin
      Key := 0;
      btnFileBrowse.Click;
   end;
end;

//------------------------------------------------------------------------------
procedure TfrmMain.FormDestroy(Sender: TObject);
//------------------------------------------------------------------------------
// Eliminate the template IDoc from memory
//------------------------------------------------------------------------------
begin
   If Assigned(IDoc) then
      IDoc.Free;
end;

end.
