{********************************************************************}
{                                                                    }
{       SAP Data Viewer                                              }
{                                                                    }
{       Copyright (c) 2004-2007 Software Union G.m.b.H.              }
{       ALL RIGHTS RESERVED                                          }
{                                                                    }
{   The entire contents of this file is protected by                 }
{   International Copyright Laws. Unauthorized reproduction,         }
{   reverse-engineering, and distribution of all or any portion of   }
{   the code contained in this file is strictly prohibited and may   }
{   result in severe civil and criminal penalties and will be        }
{   prosecuted to the maximum extent possible under the law.         }
{                                                                    }
{   RESTRICTIONS                                                     }
{                                                                    }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES            }
{   (DCU, OBJ, DLL, ETC.) ARE CONFIDENTIAL AND PROPRIETARY TRADE     }
{   SECRETS OF SOFTWARE UNION G.M.B.H. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE SAP DATA VIEWER AND ALL               }
{   ACCOMPANYING SOFTWARE LIBRARIES AS PART OF AN EXECUTABLE         }
{   PROGRAM ONLY.                                                    }
{                                                                    }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED       }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE         }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE        }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT   }
{   AND PERMISSION FROM SOFTWARE UNION G.M.B.H.                      }
{                                                                    }
{********************************************************************}
//PROFILE-NO
unit sapparse;

interface

uses sapdef;

type
   TSapParse = class
   private
   public
      constructor Create;
      destructor Destroy; override;
      procedure ReadFromFile(AFileName : string; AIDoc : TIDoc);
   end;

implementation

uses classes, sysutils, strutils;

{ TSapParse }
constructor TSapParse.Create;
begin
   inherited;
end;

destructor TSapParse.Destroy;
begin
  inherited;
end;

//------------------------------------------------------------------------------
procedure TSapParse.ReadFromFile(AFileName: string; AIDoc : TIDoc);
//------------------------------------------------------------------------------
// Quick and dirty parser for SAP IDOC structure definition files in par Format
//------------------------------------------------------------------------------

var
   InLines : TStringList;
   i, FieldLen : integer;
   sName, sLine, sType : string;
   FieldType : TIDocFieldType;
   Valid : boolean;
begin
   InLines  := TStringList.Create;
   try
      Inlines.LoadFromFile(AFileName);
      Valid := true;
      for i := 0 to InLines.Count -1 do
      begin
         sLine := UpperCase(Trim(InLines[i]));

         if copy(sLine,1,17) = 'BEGIN_DATA_RECORD' then
         begin
            Valid := false;
         end
         else
         if copy(sLine,1,20) = 'BEGIN_CONTROL_RECORD' then
         begin
            AIDoc.Add(TIDocRecord.Create('EDI_DC40', false));
            Valid := true;
         end
         else
         if copy(sLine,1,14) = 'BEGIN_SEGMENT ' then
         begin
            sName := Trim(Copy(sLine, 15, 50));
            AIDoc.Add(TIDocRecord.Create(sName, sName <> 'EDI_DC40'));
            Valid := true;
         end
         else
         if not valid then
         begin
         end
         else
         if copy(sLine,1,5) = 'NAME ' then
         begin
            sName := Trim(copy(sLine, 5, 30));
         end
         else
         if copy(sLine,1,5) = 'TYPE ' then
         begin
            sType := Trim(copy(sLine, 5, 30));
            if sType = 'CHARACTER' then
               FieldType := ftString
            else if copy(sType,1,4) = 'DATS' then
               FieldType := ftDate
            else if copy(sType,1,4) = 'TIMS' then
               FieldType := ftTime
            else
               FieldType := ftString;
         end
         else
         if copy(sLine,1,7) = 'LENGTH ' then
         begin
            FieldLen := StrToIntDef(Trim(copy(sLine,7, 30)),0);
            AIDoc.Segments[AIDoc.RecordCount-1].Add(TIDocField.Create(sName, FieldLen, FieldType));
         end
      end;
   except
      on e : exception do
         raise Exception.CreateFmt('Parsing Error. File %s line %s (%d)'#13'%s', [AFileName, sLine, i+1, e.Message]);
   end;
   InLines.Free;
end;

end.
